@@ -18,3 +18,13 @@ bazbar f = proc a ->
        -<
        a
 
+foo =
+  proc x -> do
+    returnA -< x
+   1
+
+foo a =
+  proc x ->
+    case Left x of
+      Left x -> returnA -< x
+     a
