@@ -13,6 +13,3 @@ multiline
         bar
         baz
     ) = True
-
--- https://github.com/tweag/ormolu/issues/343
-foo = (f -> 4)
