@@ -8,3 +8,6 @@ bar =
     2,
     3
   #)
+
+unit :: (# #)
+unit = (# #)
