@@ -5,3 +5,6 @@ bar = (#
     1,
     2,
     3 #)
+
+unit :: (##)
+unit = (##)
