@@ -5,3 +5,6 @@ import "a" Ab
 import "c" Ba
 import D
 import "b" Bb
+
+import "this" Y
+import "zzzz" Z
