@@ -1,4 +1,3 @@
-{-# LANGUAGE BangPatterns #-}
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE RecordWildCards #-}
 
@@ -25,9 +24,12 @@ module Ormolu
     CabalUtils.CabalInfo (..),
     CabalUtils.getCabalInfoForSourceFile,
 
-    -- * Fixity overrides
-    FixityMap,
-    getFixityOverridesForSourceFile,
+    -- * Fixity overrides and module re-exports
+    FixityOverrides,
+    defaultFixityOverrides,
+    ModuleReexports,
+    defaultModuleReexports,
+    getDotOrmoluForSourceFile,
 
     -- * Working with exceptions
     OrmoluException (..),
@@ -38,25 +40,26 @@ where
 import Control.Exception
 import Control.Monad
 import Control.Monad.IO.Class (MonadIO (..))
-import qualified Data.Map.Strict as Map
-import qualified Data.Set as Set
+import Data.Map.Strict qualified as Map
+import Data.Maybe (fromMaybe)
+import Data.Set qualified as Set
 import Data.Text (Text)
-import qualified Data.Text as T
+import Data.Text qualified as T
 import Debug.Trace
-import qualified GHC.Driver.CmdLine as GHC
-import qualified GHC.Types.SrcLoc as GHC
+import GHC.Driver.CmdLine qualified as GHC
+import GHC.Types.SrcLoc
 import Ormolu.Config
 import Ormolu.Diff.ParseResult
 import Ormolu.Diff.Text
 import Ormolu.Exception
 import Ormolu.Fixity
 import Ormolu.Parser
-import Ormolu.Parser.CommentStream (showCommentStream)
+import Ormolu.Parser.CommentStream (CommentStream (..))
 import Ormolu.Parser.Result
 import Ormolu.Printer
 import Ormolu.Utils (showOutputable)
-import qualified Ormolu.Utils.Cabal as CabalUtils
-import Ormolu.Utils.Fixity (getFixityOverridesForSourceFile)
+import Ormolu.Utils.Cabal qualified as CabalUtils
+import Ormolu.Utils.Fixity (getDotOrmoluForSourceFile)
 import Ormolu.Utils.IO
 import System.FilePath
 
@@ -86,19 +89,20 @@ ormolu cfgWithIndices path originalInput = do
   let totalLines = length (T.lines originalInput)
       cfg = regionIndicesToDeltas totalLines <$> cfgWithIndices
       fixityMap =
-        -- It is important to keep all arguments (but last) of
-        -- 'buildFixityMap' constant (such as 'defaultStrategyThreshold'),
-        -- otherwise it is going to break memoization.
-        buildFixityMap
-          defaultStrategyThreshold
-          (cfgDependencies cfg) -- memoized on the set of dependencies
+        packageFixityMap
+          (overapproximatedDependencies cfg) -- memoized on the set of dependencies
+  when (cfgDebug cfg) $ do
+    traceM $ unwords ["*** CONFIG ***", show cfg]
   (warnings, result0) <-
     parseModule' cfg fixityMap OrmoluParsingFailed path originalInput
   when (cfgDebug cfg) $ do
-    traceM "warnings:\n"
-    traceM (concatMap showWarn warnings)
+    forM_ warnings $ \(GHC.Warn reason (L loc msg)) ->
+      traceM $ unwords ["*** WARNING ***", showOutputable loc, msg, showOutputable reason]
     forM_ result0 $ \case
-      ParsedSnippet r -> traceM . showCommentStream . prCommentStream $ r
+      ParsedSnippet r -> do
+        let CommentStream comments = prCommentStream r
+        forM_ comments $ \(L loc comment) ->
+          traceM $ unwords ["*** COMMENT ***", showOutputable loc, show comment]
       _ -> pure ()
   -- We're forcing 'formattedText' here because otherwise errors (such as
   -- messages about not-yet-supported functionality) will be thrown later
@@ -181,32 +185,47 @@ refineConfig ::
   -- | Cabal info for the file, if available
   Maybe CabalUtils.CabalInfo ->
   -- | Fixity overrides, if available
-  Maybe FixityMap ->
+  Maybe FixityOverrides ->
+  -- | Module re-exports, if available
+  Maybe ModuleReexports ->
   -- | 'Config' to refine
   Config region ->
   -- | Refined 'Config'
   Config region
-refineConfig sourceType mcabalInfo mfixityOverrides rawConfig =
+refineConfig sourceType mcabalInfo mfixityOverrides mreexports rawConfig =
   rawConfig
     { cfgDynOptions = cfgDynOptions rawConfig ++ dynOptsFromCabal,
       cfgFixityOverrides =
-        Map.unionWith (<>) (cfgFixityOverrides rawConfig) fixityOverrides,
+        FixityOverrides $
+          Map.unions
+            [ unFixityOverrides fixityOverrides,
+              unFixityOverrides (cfgFixityOverrides rawConfig),
+              unFixityOverrides defaultFixityOverrides
+            ],
+      cfgModuleReexports =
+        ModuleReexports $
+          Map.unionsWith
+            (<>)
+            [ unModuleReexports reexports,
+              unModuleReexports (cfgModuleReexports rawConfig),
+              unModuleReexports defaultModuleReexports
+            ],
       cfgDependencies =
         Set.union (cfgDependencies rawConfig) depsFromCabal,
       cfgSourceType = sourceType
     }
   where
-    fixityOverrides =
-      case mfixityOverrides of
-        Nothing -> Map.empty
-        Just x -> x
+    fixityOverrides = fromMaybe defaultFixityOverrides mfixityOverrides
+    reexports = fromMaybe defaultModuleReexports mreexports
     (dynOptsFromCabal, depsFromCabal) =
       case mcabalInfo of
-        Nothing -> ([], Set.empty)
+        Nothing ->
+          -- If no cabal info is provided, assume base as a dependency by
+          -- default.
+          ([], defaultDependencies)
         Just CabalUtils.CabalInfo {..} ->
           -- It makes sense to take into account the operator info for the
-          -- package itself if we know it, as if it were its own
-          -- dependency.
+          -- package itself if we know it, as if it were its own dependency.
           (ciDynOpts, Set.insert ciPackageName ciDependencies)
 
 ----------------------------------------------------------------------------
@@ -218,9 +237,9 @@ parseModule' ::
   -- | Ormolu configuration
   Config RegionDeltas ->
   -- | Fixity Map for operators
-  LazyFixityMap ->
+  PackageFixityMap ->
   -- | How to obtain 'OrmoluException' to throw when parsing fails
-  (GHC.SrcSpan -> String -> OrmoluException) ->
+  (SrcSpan -> String -> OrmoluException) ->
   -- | File name to use in errors
   FilePath ->
   -- | Actual input for the parser
@@ -232,14 +251,6 @@ parseModule' cfg fixityMap mkException path str = do
     Left (spn, err) -> liftIO $ throwIO (mkException spn err)
     Right x -> return (warnings, x)
 
--- | Pretty-print a 'GHC.Warn'.
-showWarn :: GHC.Warn -> String
-showWarn (GHC.Warn reason l) =
-  unlines
-    [ showOutputable reason,
-      showOutputable l
-    ]
-
 -- | Detect 'SourceType' based on the file extension.
 detectSourceType :: FilePath -> SourceType
 detectSourceType mpath =
