@@ -1,9 +1,5 @@
-{-# LANGUAGE DeriveDataTypeable #-}
-{-# LANGUAGE GeneralizedNewtypeDeriving #-}
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE OverloadedStrings #-}
-{-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TypeApplications #-}
 {-# LANGUAGE ViewPatterns #-}
 
 -- | Functions for working with comment stream.
@@ -11,9 +7,9 @@ module Ormolu.Parser.CommentStream
   ( -- * Comment stream
     CommentStream (..),
     mkCommentStream,
-    showCommentStream,
 
     -- * Comment
+    LComment,
     Comment (..),
     unComment,
     hasAtomsBefore,
@@ -26,29 +22,29 @@ import Data.Char (isSpace)
 import Data.Data (Data)
 import Data.Generics.Schemes
 import Data.List.NonEmpty (NonEmpty (..))
-import qualified Data.List.NonEmpty as NE
-import qualified Data.Map.Lazy as M
+import Data.List.NonEmpty qualified as NE
+import Data.Map.Lazy qualified as M
 import Data.Maybe
-import qualified Data.Set as S
+import Data.Set qualified as S
 import Data.Text (Text)
-import qualified Data.Text as T
-import qualified GHC.Data.Strict as Strict
+import Data.Text qualified as T
+import GHC.Data.Strict qualified as Strict
 import GHC.Hs (HsModule)
 import GHC.Hs.Doc
 import GHC.Hs.Extension
 import GHC.Hs.ImpExp
 import GHC.Parser.Annotation (EpAnnComments (..), getLocA)
-import qualified GHC.Parser.Annotation as GHC
+import GHC.Parser.Annotation qualified as GHC
 import GHC.Types.SrcLoc
 import Ormolu.Parser.Pragma
-import Ormolu.Utils (onTheSameLine, showOutputable)
+import Ormolu.Utils (onTheSameLine)
 
 ----------------------------------------------------------------------------
 -- Comment stream
 
 -- | A stream of 'RealLocated' 'Comment's in ascending order with respect to
 -- beginning of corresponding spans.
-newtype CommentStream = CommentStream [RealLocated Comment]
+newtype CommentStream = CommentStream [LComment]
   deriving (Eq, Data, Semigroup, Monoid)
 
 -- | Create 'CommentStream' from 'HsModule'. The pragmas are
@@ -57,10 +53,10 @@ mkCommentStream ::
   -- | Original input
   Text ->
   -- | Module to use for comment extraction
-  HsModule ->
+  HsModule GhcPs ->
   -- | Stack header, pragmas, and comment stream
-  ( Maybe (RealLocated Comment),
-    [([RealLocated Comment], Pragma)],
+  ( Maybe LComment,
+    [([LComment], Pragma)],
     CommentStream
   )
 mkCommentStream input hsModule =
@@ -108,17 +104,11 @@ mkCommentStream input hsModule =
     only :: a -> Bool
     only _ = True
 
--- | Pretty-print a 'CommentStream'.
-showCommentStream :: CommentStream -> String
-showCommentStream (CommentStream xs) =
-  unlines $
-    showComment <$> xs
-  where
-    showComment (L l str) = showOutputable l ++ " " ++ show str
-
 ----------------------------------------------------------------------------
 -- Comment
 
+type LComment = RealLocated Comment
+
 -- | A wrapper for a single comment. The 'Bool' indicates whether there were
 -- atoms before beginning of the comment in the original input. The
 -- 'NonEmpty' list inside contains lines of multiline comment @{- … -}@ or
@@ -135,7 +125,7 @@ mkComment ::
   -- | Raw comment string
   RealLocated Text ->
   -- | Remaining lines of original input and the constructed 'Comment'
-  ([(Int, Text)], RealLocated Comment)
+  ([(Int, Text)], LComment)
 mkComment ls (L l s) = (ls', comment)
   where
     comment =
@@ -189,7 +179,7 @@ isMultilineComment (Comment _ (x :| _)) = "{-" `T.isPrefixOf` x
 extractStackHeader ::
   -- | Comment stream to analyze
   [RealLocated Text] ->
-  ([RealLocated Text], Maybe (RealLocated Comment))
+  ([RealLocated Text], Maybe LComment)
 extractStackHeader = \case
   [] -> ([], Nothing)
   (x : xs) ->
@@ -207,7 +197,7 @@ extractPragmas ::
   Text ->
   -- | Comment stream to analyze
   [RealLocated Text] ->
-  ([RealLocated Comment], [([RealLocated Comment], Pragma)])
+  ([LComment], [([LComment], Pragma)])
 extractPragmas input = go initialLs id id
   where
     initialLs = zip [1 ..] (T.lines input)
