@@ -1,19 +1,16 @@
 {-# LANGUAGE LambdaCase #-}
-{-# LANGUAGE NamedFieldPuns #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
-{-# LANGUAGE ScopedTypeVariables #-}
-{-# LANGUAGE TypeApplications #-}
 {-# LANGUAGE TypeFamilies #-}
-{-# LANGUAGE TypeOperators #-}
 {-# LANGUAGE ViewPatterns #-}
 
 module Ormolu.Printer.Meat.Declaration.Value
   ( p_valDecl,
     p_pat,
     p_hsExpr,
-    p_hsSplice,
+    p_hsUntypedSplice,
     p_stringLit,
+    IsApplicand (..),
     p_hsExpr',
     p_hsCmdTop,
     exprPlacement,
@@ -30,14 +27,13 @@ import Data.Functor ((<&>))
 import Data.Generics.Schemes (everything)
 import Data.List (intersperse, sortBy)
 import Data.List.NonEmpty (NonEmpty (..), (<|))
-import qualified Data.List.NonEmpty as NE
+import Data.List.NonEmpty qualified as NE
 import Data.Maybe
 import Data.Text (Text)
-import qualified Data.Text as Text
+import Data.Text qualified as Text
 import Data.Void
 import GHC.Data.Bag (bagToList)
-import GHC.Data.FastString (FastString, lengthFS)
-import qualified GHC.Data.Strict as Strict
+import GHC.Data.Strict qualified as Strict
 import GHC.Hs
 import GHC.LanguageExtensions.Type (Extension (NegativeLiterals))
 import GHC.Parser.CharClass (is_space)
@@ -46,6 +42,7 @@ import GHC.Types.Fixity
 import GHC.Types.Name.Reader
 import GHC.Types.SourceText
 import GHC.Types.SrcLoc
+import Language.Haskell.Syntax.Basic
 import Ormolu.Printer.Combinators
 import Ormolu.Printer.Meat.Common
 import {-# SOURCE #-} Ormolu.Printer.Meat.Declaration
@@ -70,8 +67,8 @@ data GroupStyle
 
 p_valDecl :: HsBind GhcPs -> R ()
 p_valDecl = \case
-  FunBind _ funId funMatches _ -> p_funBind funId funMatches
-  PatBind _ pat grhss _ -> p_match PatternBind False NoSrcStrict [pat] grhss
+  FunBind _ funId funMatches -> p_funBind funId funMatches
+  PatBind _ pat grhss -> p_match PatternBind False NoSrcStrict [pat] grhss
   VarBind {} -> notImplemented "VarBinds" -- introduced by the type checker
   PatSynBind _ psb -> p_patSynBind psb
 
@@ -190,7 +187,7 @@ p_match' placer render style isInfix strictness m_pats GRHSs {..} = do
       False <$ case style of
         Function name -> p_rdrName name
         _ -> return ()
-    Just ne_pats -> do
+    Just ne_pats@(head_pat :| tail_pats) -> do
       let combinedSpans = case style of
             Function name -> combineSrcSpans (getLocA name) patSpans
             _ -> patSpans
@@ -208,7 +205,7 @@ p_match' placer render style isInfix strictness m_pats GRHSs {..} = do
           PatternBind -> stdCase
           Case -> stdCase
           Lambda -> do
-            let needsSpace = case unLoc (NE.head ne_pats) of
+            let needsSpace = case unLoc head_pat of
                   LazyPat _ _ -> True
                   BangPat _ _ -> True
                   SplicePat _ _ -> True
@@ -216,7 +213,13 @@ p_match' placer render style isInfix strictness m_pats GRHSs {..} = do
             txt "\\"
             when needsSpace space
             sitcc stdCase
-          LambdaCase -> stdCase
+          LambdaCase -> do
+            located' p_pat head_pat
+            unless (null tail_pats) $ do
+              breakpoint
+              -- When we have multiple patterns (with `\cases`) across multiple
+              -- lines, we have to indent all but the first pattern.
+              inci $ sep breakpoint (located' p_pat) tail_pats
       return indentBody
   let -- Calculate position of end of patterns. This is useful when we decide
       -- about putting certain constructions in hanging positions.
@@ -321,10 +324,10 @@ p_grhs' parentPlacement placer render style (GRHS _ guards body) =
     p_body = located body render
 
 p_hsCmd :: HsCmd GhcPs -> R ()
-p_hsCmd = p_hsCmd' N
+p_hsCmd = p_hsCmd' NotApplicand N
 
-p_hsCmd' :: BracketStyle -> HsCmd GhcPs -> R ()
-p_hsCmd' s = \case
+p_hsCmd' :: IsApplicand -> BracketStyle -> HsCmd GhcPs -> R ()
+p_hsCmd' isApp s = \case
   HsCmdArrApp _ body input arrType rightToLeft -> do
     let (l, r) = if rightToLeft then (body, input) else (input, body)
     located l p_hsExpr
@@ -343,34 +346,33 @@ p_hsCmd' s = \case
       breakpoint
       inci (sequence_ (intersperse breakpoint (located' (p_hsCmdTop N) <$> cmds)))
   HsCmdArrForm _ form Infix _ [left, right] -> do
-    fixityOverrides <- askFixityOverrides
-    fixityMap <- askFixityMap
+    modFixityMap <- askModuleFixityMap
     let opTree = OpBranches [cmdOpTree left, cmdOpTree right] [form]
     p_cmdOpTree
       s
-      (reassociateOpTree (getOpName . unLoc) fixityOverrides fixityMap opTree)
+      (reassociateOpTree (getOpName . unLoc) modFixityMap opTree)
   HsCmdArrForm _ _ Infix _ _ -> notImplemented "HsCmdArrForm"
   HsCmdApp _ cmd expr -> do
-    located cmd (p_hsCmd' s)
-    space
-    located expr p_hsExpr
+    located cmd (p_hsCmd' Applicand s)
+    breakpoint
+    inci $ located expr p_hsExpr
   HsCmdLam _ mgroup -> p_matchGroup' cmdPlacement p_hsCmd Lambda mgroup
   HsCmdPar _ _ c _ -> parens N (located c p_hsCmd)
   HsCmdCase _ e mgroup ->
-    p_case cmdPlacement p_hsCmd e mgroup
+    p_case isApp cmdPlacement p_hsCmd e mgroup
   HsCmdLamCase _ variant mgroup ->
-    p_lamcase variant cmdPlacement p_hsCmd mgroup
+    p_lamcase isApp variant cmdPlacement p_hsCmd mgroup
   HsCmdIf _ _ if' then' else' ->
     p_if cmdPlacement p_hsCmd if' then' else'
   HsCmdLet _ _ localBinds _ c ->
     p_let p_hsCmd localBinds c
   HsCmdDo _ es -> do
     txt "do"
-    p_stmts cmdPlacement (p_hsCmd' S) es
+    p_stmts isApp cmdPlacement (p_hsCmd' NotApplicand S) es
 
 -- | Print a top-level command.
 p_hsCmdTop :: BracketStyle -> HsCmdTop GhcPs -> R ()
-p_hsCmdTop s (HsCmdTop _ cmd) = located cmd (p_hsCmd' s)
+p_hsCmdTop s (HsCmdTop _ cmd) = located cmd (p_hsCmd' NotApplicand s)
 
 -- | Render an expression preserving blank lines between such consecutive
 -- expressions found in the original source code.
@@ -476,6 +478,7 @@ p_stmts ::
   ( Anno (Stmt GhcPs (LocatedA body)) ~ SrcSpanAnnA,
     Anno [LocatedA (Stmt GhcPs (LocatedA body))] ~ SrcSpanAnnL
   ) =>
+  IsApplicand ->
   -- | Placer
   (body -> Placement) ->
   -- | Render
@@ -483,12 +486,21 @@ p_stmts ::
   -- | Statements to render
   LocatedL [LocatedA (Stmt GhcPs (LocatedA body))] ->
   R ()
-p_stmts placer render es = do
+p_stmts isApp placer render es = do
   breakpoint
   ub <- layoutToBraces <$> getLayout
-  inci . located es $
-    sepSemi
-      (ub . withSpacing (p_stmt' placer render))
+  let p_stmtExt (relPos, stmt) =
+        ub' $ withSpacing (p_stmt' placer render) stmt
+        where
+          -- We need to set brace usage information for all but the last
+          -- statement (e.g.in the case of nested do blocks).
+          ub' = case relPos of
+            FirstPos -> ub
+            MiddlePos -> ub
+            LastPos -> id
+            SinglePos -> id
+  inciApplicand isApp . located es $
+    sepSemi p_stmtExt . attachRelativePos
 
 gatherStmt :: ExprLStmt GhcPs -> [[ExprLStmt GhcPs]]
 gatherStmt (L _ (ParStmt _ block _ _)) =
@@ -545,21 +557,8 @@ p_hsLocalBinds = \case
       _ -> id
 
 p_ldotFieldOcc :: XRec GhcPs (DotFieldOcc GhcPs) -> R ()
-p_ldotFieldOcc = located' $ p_lFieldLabelString . dfoLabel
-  where
-    p_lFieldLabelString (L (locA -> s) fs) = parensIfOp . atom @FastString $ fs
-      where
-        -- HACK For OverloadedRecordUpdate:
-        -- In operator field updates (i.e. `f {(+) = 1}`), we don't have
-        -- information whether parens are necessary. As a workaround,
-        -- we look if the RealSrcSpan is bigger than the string fs.
-        parensIfOp
-          | isOneLineSpan s,
-            Just realS <- srcSpanToRealSrcSpan s,
-            let spanLength = srcSpanEndCol realS - srcSpanStartCol realS,
-            lengthFS fs < spanLength =
-              parens N
-          | otherwise = id
+p_ldotFieldOcc =
+  located' $ p_rdrName . fmap (mkVarUnqual . field_label) . dfoLabel
 
 p_ldotFieldOccs :: [XRec GhcPs (DotFieldOcc GhcPs)] -> R ()
 p_ldotFieldOccs = sep (txt ".") p_ldotFieldOcc
@@ -584,16 +583,30 @@ p_hsFieldBind p_lhs HsFieldBind {..} = do
     placeHanging placement (located hfbRHS p_hsExpr)
 
 p_hsExpr :: HsExpr GhcPs -> R ()
-p_hsExpr = p_hsExpr' N
+p_hsExpr = p_hsExpr' NotApplicand N
 
-p_hsExpr' :: BracketStyle -> HsExpr GhcPs -> R ()
-p_hsExpr' s = \case
+-- | An applicand is the left-hand side in a function application, i.e. @f@ in
+-- @f a@. We need to track this in order to add extra identation in cases like
+--
+-- > foo =
+-- >   do
+-- >       succ
+-- >     1
+data IsApplicand = Applicand | NotApplicand
+
+inciApplicand :: IsApplicand -> R () -> R ()
+inciApplicand = \case
+  Applicand -> inci . inci
+  NotApplicand -> inci
+
+p_hsExpr' :: IsApplicand -> BracketStyle -> HsExpr GhcPs -> R ()
+p_hsExpr' isApp s = \case
   HsVar _ name -> p_rdrName name
   HsUnboundVar _ occ -> atom occ
   HsRecSel _ fldOcc -> p_fieldOcc fldOcc
-  HsOverLabel _ v -> do
+  HsOverLabel _ sourceText _ -> do
     txt "#"
-    atom v
+    p_sourceText sourceText
   HsIPVar _ (HsIPName name) -> do
     txt "?"
     atom name
@@ -606,7 +619,7 @@ p_hsExpr' s = \case
   HsLam _ mgroup ->
     p_matchGroup Lambda mgroup
   HsLamCase _ variant mgroup ->
-    p_lamcase variant exprPlacement p_hsExpr mgroup
+    p_lamcase isApp variant exprPlacement p_hsExpr mgroup
   HsApp _ f x -> do
     let -- In order to format function applications with multiple parameters
         -- nicer, traverse the AST to gather the function and all the
@@ -634,40 +647,25 @@ p_hsExpr' s = \case
     -- one.
     case placement of
       Normal -> do
-        let -- Usually we want to bump indentation for arguments for the
-            -- sake of readability. However:
-            -- When the function is itself a multi line do-block or a case
-            -- expression, we can't indent by indentStep or more.
-            -- When we are on the other hand *in* a do block, we have to
-            -- indent by at least 1.
-            -- Thus, we indent by half of indentStep when the function is
-            -- a multi line do block or case expression.
-            indentArg
-              | isOneLineSpan (getLocA func) = inci
-              | otherwise = case unLoc func of
-                  HsDo {} -> inciHalf
-                  HsCase {} -> inciHalf
-                  HsLamCase {} -> inciHalf
-                  _ -> inci
         ub <-
           getLayout <&> \case
             SingleLine -> useBraces
             MultiLine -> id
         ub $ do
-          located func (p_hsExpr' s)
+          located func (p_hsExpr' Applicand s)
           breakpoint
-          indentArg $ sep breakpoint (located' p_hsExpr) initp
-        indentArg $ do
+          inci $ sep breakpoint (located' p_hsExpr) initp
+        inci $ do
           unless (null initp) breakpoint
           located lastp p_hsExpr
       Hanging -> do
         useBraces . switchLayout [initSpan] $ do
-          located func (p_hsExpr' s)
+          located func (p_hsExpr' Applicand s)
           breakpoint
           sep breakpoint (located' p_hsExpr) initp
-        placeHanging placement . dontUseBraces $
+        placeHanging placement $
           located lastp p_hsExpr
-  HsAppType _ e a -> do
+  HsAppType _ e _ a -> do
     located e p_hsExpr
     breakpoint
     inci $ do
@@ -679,12 +677,11 @@ p_hsExpr' s = \case
         _ -> return ()
       located (hswc_body a) p_hsType
   OpApp _ x op y -> do
-    fixityOverrides <- askFixityOverrides
-    fixityMap <- askFixityMap
+    modFixityMap <- askModuleFixityMap
     let opTree = OpBranches [exprOpTree x, exprOpTree y] [op]
     p_exprOpTree
       s
-      (reassociateOpTree (getOpName . unLoc) fixityOverrides fixityMap opTree)
+      (reassociateOpTree (getOpName . unLoc) modFixityMap opTree)
   NegApp _ e _ -> do
     negativeLiterals <- isExtensionEnabled NegativeLiterals
     let isLiteral = case unLoc e of
@@ -721,7 +718,7 @@ p_hsExpr' s = \case
             Unboxed -> parensHash
     enclSpan <-
       fmap (flip RealSrcSpan Strict.Nothing) . maybeToList
-        <$> getEnclosingSpan (const True)
+        <$> getEnclosingSpan
     if isSection
       then
         switchLayout [] . parens' s $
@@ -732,20 +729,20 @@ p_hsExpr' s = \case
   ExplicitSum _ tag arity e ->
     p_unboxedSum N tag arity (located e p_hsExpr)
   HsCase _ e mgroup ->
-    p_case exprPlacement p_hsExpr e mgroup
+    p_case isApp exprPlacement p_hsExpr e mgroup
   HsIf _ if' then' else' ->
     p_if exprPlacement p_hsExpr if' then' else'
   HsMultiIf _ guards -> do
     txt "if"
     breakpoint
-    inci . inci $ sep newline (located' (p_grhs RightArrow)) guards
+    inciApplicand isApp $ sep newline (located' (p_grhs RightArrow)) guards
   HsLet _ _ localBinds _ e ->
     p_let p_hsExpr localBinds e
   HsDo _ doFlavor es -> do
     let doBody moduleName header = do
           forM_ moduleName $ \m -> atom m *> txt "."
           txt header
-          p_stmts exprPlacement (p_hsExpr' S) es
+          p_stmts isApp exprPlacement (p_hsExpr' NotApplicand S) es
         compBody = brackets s . located es $ \xs -> do
           let p_parBody =
                 sep
@@ -841,7 +838,8 @@ p_hsExpr' s = \case
     breakpoint'
     txt "||]"
   HsUntypedBracket epAnn x -> p_hsQuote epAnn x
-  HsSpliceE _ splice -> p_hsSplice splice
+  HsTypedSplice _ expr -> p_hsSpliceTH True expr DollarSplice
+  HsUntypedSplice _ untySplice -> p_hsUntypedSplice DollarSplice untySplice
   HsProc _ p e -> do
     txt "proc"
     located p $ \x -> do
@@ -856,7 +854,7 @@ p_hsExpr' s = \case
     breakpoint
     inci (located e p_hsExpr)
   HsPragE _ prag x -> case prag of
-    HsPragSCC _ _ name -> do
+    HsPragSCC _ name -> do
       txt "{-# SCC "
       atom name
       txt " #-}"
@@ -927,6 +925,7 @@ p_case ::
   ( Anno (GRHS GhcPs (LocatedA body)) ~ SrcAnn NoEpAnns,
     Anno (Match GhcPs (LocatedA body)) ~ SrcSpanAnnA
   ) =>
+  IsApplicand ->
   -- | Placer
   (body -> Placement) ->
   -- | Render
@@ -936,19 +935,20 @@ p_case ::
   -- | Match group
   MatchGroup GhcPs (LocatedA body) ->
   R ()
-p_case placer render e mgroup = do
+p_case isApp placer render e mgroup = do
   txt "case"
   space
   located e p_hsExpr
   space
   txt "of"
   breakpoint
-  inci (p_matchGroup' placer render Case mgroup)
+  inciApplicand isApp (p_matchGroup' placer render Case mgroup)
 
 p_lamcase ::
   ( Anno (GRHS GhcPs (LocatedA body)) ~ SrcAnn NoEpAnns,
     Anno (Match GhcPs (LocatedA body)) ~ SrcSpanAnnA
   ) =>
+  IsApplicand ->
   -- | Variant (@\\case@ or @\\cases@)
   LamCaseVariant ->
   -- | Placer
@@ -958,12 +958,12 @@ p_lamcase ::
   -- | Expression
   MatchGroup GhcPs (LocatedA body) ->
   R ()
-p_lamcase variant placer render mgroup = do
+p_lamcase isApp variant placer render mgroup = do
   txt $ case variant of
     LamCase -> "\\case"
     LamCases -> "\\cases"
   breakpoint
-  inci (p_matchGroup' placer render LambdaCase mgroup)
+  inciApplicand isApp (p_matchGroup' placer render LambdaCase mgroup)
 
 p_if ::
   -- | Placer
@@ -1015,7 +1015,7 @@ p_pat = \case
   LazyPat _ pat -> do
     txt "~"
     located pat p_pat
-  AsPat _ name pat -> do
+  AsPat _ name _ pat -> do
     p_rdrName name
     txt "@"
     located pat p_pat
@@ -1040,7 +1040,7 @@ p_pat = \case
         p_rdrName pat
         unless (null tys && null xs) breakpoint
         inci . sitcc $
-          sep breakpoint (sitcc . either p_hsPatSigType (located' p_pat)) $
+          sep breakpoint (sitcc . either p_hsConPatTyArg (located' p_pat)) $
             (Left <$> tys) <> (Right <$> xs)
       RecCon (HsRecFields fields dotdot) -> do
         p_rdrName pat
@@ -1051,7 +1051,7 @@ p_pat = \case
         inci . braces N . sep commaDel f $
           case dotdot of
             Nothing -> Just <$> fields
-            Just (L _ n) -> (Just <$> take n fields) ++ [Nothing]
+            Just (L _ (RecFieldsDotDot n)) -> (Just <$> take n fields) ++ [Nothing]
       InfixCon l r -> do
         switchLayout [getLocA l, getLocA r] $ do
           located l p_pat
@@ -1066,7 +1066,7 @@ p_pat = \case
     txt "->"
     breakpoint
     inci (located pat p_pat)
-  SplicePat _ splice -> p_hsSplice splice
+  SplicePat _ splice -> p_hsUntypedSplice DollarSplice splice
   LitPat _ p -> atom p
   NPat _ v (isJust -> isNegated) _ -> do
     when isNegated $ do
@@ -1088,6 +1088,9 @@ p_pat = \case
 p_hsPatSigType :: HsPatSigType GhcPs -> R ()
 p_hsPatSigType (HsPS _ ty) = txt "@" *> located ty p_hsType
 
+p_hsConPatTyArg :: HsConPatTyArg GhcPs -> R ()
+p_hsConPatTyArg (HsConPatTyArg _ patSigTy) = p_hsPatSigType patSigTy
+
 p_pat_hsFieldBind :: HsRecField GhcPs (LPat GhcPs) -> R ()
 p_pat_hsFieldBind HsFieldBind {..} = do
   located hfbLHS p_fieldOcc
@@ -1112,11 +1115,10 @@ p_unboxedSum s tag arity m = do
             space
   parensHash s $ sep (txt "|") f args
 
-p_hsSplice :: HsSplice GhcPs -> R ()
-p_hsSplice = \case
-  HsTypedSplice _ deco _ expr -> p_hsSpliceTH True expr deco
-  HsUntypedSplice _ deco _ expr -> p_hsSpliceTH False expr deco
-  HsQuasiQuote _ _ quoterName _ str -> do
+p_hsUntypedSplice :: SpliceDecoration -> HsUntypedSplice GhcPs -> R ()
+p_hsUntypedSplice deco = \case
+  HsUntypedSpliceExpr _ expr -> p_hsSpliceTH False expr deco
+  HsQuasiQuote _ quoterName str -> do
     txt "["
     p_rdrName (noLocA quoterName)
     txt "|"
@@ -1124,7 +1126,6 @@ p_hsSplice = \case
     -- formatting here without potentially breaking someone's code.
     atom str
     txt "|]"
-  HsSpliced {} -> notImplemented "HsSpliced"
 
 p_hsSpliceTH ::
   -- | Typed splice?
@@ -1275,7 +1276,7 @@ exprPlacement :: HsExpr GhcPs -> Placement
 exprPlacement = \case
   -- Only hang lambdas with single line parameter lists
   HsLam _ mg -> case mg of
-    MG _ (L _ [L _ (Match _ _ (x : xs) _)]) _
+    MG _ (L _ [L _ (Match _ _ (x : xs) _)])
       | isOneLineSpan (combineSrcSpans' $ fmap getLocA (x :| xs)) ->
           Hanging
     _ -> Normal
